#!/usr/bin/php
<?php
   /**
    * $Id$
    *
    * Cloudfiles Backup to S3
    *
    * Uses Rackspace CloudFiles PHP API to copy entire container contents (even containers
    * > 10000 objects to Amazon S3 account. Backups are done in an 'rsync'-fashion (only
    * objects which differ are sent to Amazon -- saving on bandwidth costs).
    *
    * LIMITATIONS:
    * 1) Because we're reading each CloudFiles object into memory, your CloudServer will need
    *    to have a memory footprint large enough to handle your largest file.
    * 2) You MUST change the CloudFiles 'CF_Object' to expose "etag" as a public (not private) var.
    *    Change Line 1592 of 'cloudfiles.php' to:
    *    public $etag;
    *
    * REQUIRES:
    * Rackspace CloudFiles PHP API:
    * http://github.com/pas256/Rackspace-Cloud-PHP-Library)
    * Amazon S3 PHP Class:
    * http://code.google.com/p/amazon-s3-php-class/
    *
    * USE:
    * Just populate your these varaibles for your access to Rackspace & Amazon:
    * $rackUser => your rackspace User
    * $rackAPIKey => your rackspace Password
    * $rackContainer => rackspace container you want to back up
    * $amznAPIKey => Amazon API Key
    * $amznAPISecret => 'Secret' Amazon API Key
    *
    * Then run, and feel free to put on crontab! Enjoy!
    *
    * CREDITS:
    * Christopher Sheperd's "S3_Backup" class is utilized for the S3 uploads
    * http://www.belgo.com/media/s3/s3backup.php.txt
    *
    * @author rICh <rich@quicloud.com>
    */

$blnVerbose = false;
if(isset($argv[1]) && ($argv[1] == '-v')) {
  $blnVerbose = true;
 }

print "start run at ".date('Y-m-d H:i:s')."\n";
ini_set('memory_limit','800M');

$rackUser = 'RACKSPACE_API_USERNAME';
$rackAPIKey = 'RACKSPACE_API_KEY';
$rackContainer = 'RACKSPACE_CONTAINER_NAME';
$amznAPIKey = 'AMAZON_KEY';
$amznAPISecret = 'AMAZON_SECRECT_KEY';
$amznBucketName = 'AMAZON_BUCKET_NAME';

// Load Rackspace CloudFiles API and Amazon S3 API
require_once("/etc/cloudfiles-to-s3/includes/rackspace/cloudfiles.php");
require_once("/etc/cloudfiles-to-s3/includes/rackspace/cloudfiles_exceptions.php");
require_once("/etc/cloudfiles-to-s3/includes/rackspace/cloudfiles_http.php");
require_once("/etc/cloudfiles-to-s3/includes/amazon/S3.php");

$intLimit = $intFound = 10000; //10K object limit in Rackspace 'getContainer' call
$strMarker = '';
$intPutCount = $intSkipCount = 0;
$arrObjects = array();

// This is the CloudFiles API integration... comment out the following lines if you're using CloudFuse
$authRack = new CF_Authentication($rackUser, $rackAPIKey);
$authRack->authenticate();
$connRack = new CF_Connection($authRack);
$objContainer = $connRack->get_container($rackContainer);
$arrObjects = $objContainer->get_objects($intLimit,$strMarker);
$intFound = count($arrObjects);

print "Loaded Rackspace Container (".$intFound." Objects)\n";
S3::setAuth($amznAPIKey,$amznAPISecret);

//grab all the hashes for the bucket ( so we can compare later)
$arrBucketInfo = S3::getBucket($amznBucketName);
//clear some memory + make lookups faster (we only need hash for each)
foreach($arrBucketInfo as $strKey => $arrVal) {
  unset($arrBucketInfo[$strKey]);
  $arrBucketInfo[$strKey] = $arrVal['hash'];
}
print "Loaded Amazon Bucket (".count($arrBucketInfo)." Objects)\n";

while($intFound == $intLimit) {
  $arrObjects = $objContainer->get_objects($intLimit,$strMarker);
  $intFound = count($arrObjects);
  $strMarker = $arrObjects[$intFound-1]->name;

  foreach($arrObjects as $objSingleFile) {
    $strObjName = $objSingleFile->name;
    $strObjContentType = $objSingleFile->content_type;
    $strObjEtag = $objSingleFile->etag;
    $arrObjMetaData = $objSingleFile->metadata;
    
    //don't send to Amazon unless hashes differ
    $amznObjHash = '';
    if(isset($arrBucketInfo[$strObjName])) {
      $amznObjHash = $arrBucketInfo[$strObjName];
    }

    if($amznObjHash == $strObjEtag) {
      $intSkipCount++;
      if($blnVerbose) {
	print "Skipping '$strObjName' (Already loaded into Amazon bucket '$amznBucketName')\n";
      }
      continue;
    } else {
      //Send up to AMZN -- only perform expensive read if we have to
      $mixObjData = $objSingleFile->read();
      S3::putObject($mixObjData,$amznBucketName,$strObjName,S3::ACL_PRIVATE,$arrObjMetaData,array('Content-Type'=>$strObjContentType));
      if($blnVerbose) {
	print "Putting '$strObjName' up to Amazon bucket '$amznBucketName'\n";
      }
      $intPutCount++;
    }
  }
}

print "Put up $intPutCount files\n";
print "Skipped $intSkipCount files\n";
print "end run at ".date('Y-m-d H:i:s')."\n";
?>
